from typing import Literal, Optional
import json
import random
import re
import io
from plotly import graph_objects as go
from plotly import io as pio
from PIL import Image
from loguru import logger

from agentlin.core.agent_schema import image_to_base64
from agentlin.code_interpreter.types import (
    MIME_MARKDOWN,
    MIME_TEXT,
    MIME_IMAGE_PNG,
    MIME_IMAGE_JPEG,
    MIME_PLOTLY,
    MIME_TABLE_V1,
    MIME_TABLE_V2,
    MIME_HTML,
    Block,
)


def delete_color_control_char(string: str) -> str:
    ansi_escape = re.compile(r"(\x9B|\x1B\[)[0-?]*[ -\/]*[@-~]")
    return ansi_escape.sub("", string)


def iopub_msg_to_msg(iopub_msg: Optional[dict]) -> Optional[dict]:
    """
    Convert iopub message to a standardized message format.
    """
    if not iopub_msg:
        return None
    if iopub_msg["msg_type"] == "stream":
        if iopub_msg["content"].get("name") == "stdout":
            output = iopub_msg["content"]["text"]
            return {
                "msg": iopub_msg,
                "type": "stdout",
                "output": delete_color_control_char(output),
            }
    elif iopub_msg["msg_type"] == "execute_result":
        if "data" in iopub_msg["content"]:
            data = iopub_msg["content"]["data"]
            output = None
            if MIME_PLOTLY in data:
                fig_json = data[MIME_PLOTLY]
                if isinstance(fig_json, dict):
                    fig = go.Figure(fig_json)
                else:
                    fig = pio.from_json(fig_json)

                # Convert to image bytes using plotly
                img_bytes = pio.to_image(fig, format="png")
                image = Image.open(io.BytesIO(img_bytes))
                base64_str = image_to_base64(image)
                return {
                    "msg": iopub_msg,
                    "type": "execute_result_plotly",
                    "output": base64_str,
                    "fig_json": fig_json,
                }
            elif MIME_TABLE_V1 in data or MIME_TABLE_V2 in data:
                return {
                    "msg": iopub_msg,
                    "type": "execute_result_table",
                    "output": data,
                }
            elif MIME_MARKDOWN in data:
                output = data[MIME_MARKDOWN]
                return {
                    "msg": iopub_msg,
                    "type": "execute_result_text",
                    "output": delete_color_control_char(output),
                }
            elif MIME_IMAGE_PNG in data:
                output = data[MIME_IMAGE_PNG]
                output = "data:image/png;base64," + output
                return {
                    "msg": iopub_msg,
                    "type": "execute_result_png",
                    "output": output,
                }
            elif MIME_IMAGE_JPEG in data:
                output = data[MIME_IMAGE_JPEG]
                output = "data:image/jpeg;base64," + output
                return {
                    "msg": iopub_msg,
                    "type": "execute_result_jpeg",
                    "output": output,
                }
            elif MIME_HTML in data:
                output = data[MIME_HTML]
                return {
                    "msg": iopub_msg,
                    "type": "execute_result_html",
                    "output": output,
                }
            elif MIME_TEXT in data:
                output = data[MIME_TEXT]
                return {
                    "msg": iopub_msg,
                    "type": "execute_result_text",
                    "output": delete_color_control_char(output),
                }
            else:
                logger.warning("content type not supported in execute_result")
                logger.warning(data)
    elif iopub_msg["msg_type"] == "display_data":
        if "data" in iopub_msg["content"]:
            data = iopub_msg["content"]["data"]
            output = None
            if MIME_PLOTLY in data:
                fig_json = data[MIME_PLOTLY]
                if isinstance(fig_json, dict):
                    fig = go.Figure(fig_json)
                else:
                    fig = pio.from_json(fig_json)

                # Convert to image bytes using plotly
                img_bytes = pio.to_image(fig, format="png")
                image = Image.open(io.BytesIO(img_bytes))
                base64_str = image_to_base64(image)
                return {
                    "msg": iopub_msg,
                    "type": "display_plotly",
                    "output": base64_str,
                    "fig_json": fig_json,
                }
            elif MIME_TABLE_V1 in data or MIME_TABLE_V2 in data:
                return {
                    "msg": iopub_msg,
                    "type": "display_result_table",
                    "output": data,
                }
            elif MIME_MARKDOWN in data:
                output = data[MIME_MARKDOWN]
                return {
                    "msg": iopub_msg,
                    "type": "display_text",
                    "output": delete_color_control_char(output),
                }
            elif MIME_IMAGE_PNG in data:
                output = data[MIME_IMAGE_PNG]
                output = "data:image/png;base64," + output
                return {
                    "msg": iopub_msg,
                    "type": "display_png",
                    "output": output,
                }
            elif MIME_IMAGE_JPEG in data:
                output = data[MIME_IMAGE_JPEG]
                output = "data:image/jpeg;base64," + output
                return {
                    "msg": iopub_msg,
                    "type": "display_jpeg",
                    "output": output,
                }
            elif MIME_HTML in data:
                output = data[MIME_HTML]
                return {
                    "msg": iopub_msg,
                    "type": "display_html",
                    "output": output,
                }
            elif MIME_TEXT in data:
                output = data[MIME_TEXT]
                return {
                    "msg": iopub_msg,
                    "type": "display_text",
                    "output": delete_color_control_char(output),
                }
            else:
                logger.warning("content type not supported in display_data")
                logger.warning(data)
    elif iopub_msg["msg_type"] == "error":
        if "traceback" in iopub_msg["content"]:
            output = "\n".join(iopub_msg["content"]["traceback"])
            return {
                "msg": iopub_msg,
                "type": "error",
                "output": delete_color_control_char(output),
            }
    return None


def parse_execution_message_list(msg_list: list[Optional[dict]]):
    all_output = list(x for x in map(iopub_msg_to_msg, msg_list) if x is not None)
    return all_output


def msg_to_message_content(msg: Optional[dict]) -> Optional[dict]:
    if not msg:
        return None
    mark = msg["type"]
    output = msg["output"]
    if mark in ("stdout", "execute_result_text", "display_text", "error"):
        return {"type": "text", "text": output}
    elif mark in ("execute_result_png", "execute_result_jpeg", "display_png", "display_jpeg"):
        return {"type": "image_url", "image_url": {"url": output}}
    elif mark in ("execute_result_plotly", "display_plotly"):
        return {"type": "image_url", "image_url": {"url": output}}
    elif mark in ("execute_result_html", "display_html"):
        return {"type": "text", "text": output}
    elif mark in ("execute_result_json", "display_json"):
        return {"type": "text", "text": json.dumps(output, ensure_ascii=False)}
    elif mark in ("execute_result_table", "display_result_table"):
        return {"type": "text", "text": output}  # TODO output 含 V1, V2
    return None


def parse_msg_list_to_message_content(msg_list: list[Optional[dict]]):
    return list(x for x in map(msg_to_message_content, msg_list) if x is not None)


def msg_to_full_block(msg: Optional[dict]):
    if not msg:
        return None
    mark = msg["type"]
    output = msg["output"]
    uid = random.randint(100000, 999999)  # Generate a random UID for the block (6 digits)
    if mark in ("stdout", "execute_result_text", "display_text", "error"):
        return {
            "type": "text",
            "text": output,
            # "id": uid,
        }
    elif mark in ("execute_result_png", "display_png", "execute_result_jpeg", "display_jpeg"):
        return {
            "type": "image_url",
            "image_url": {
                "url": output,
            },
            "id": uid,
        }
    elif mark in ("execute_result_plotly", "display_plotly"):
        fig_json = msg["fig_json"]
        return {
            "type": "plotly-json",
            "data": {
                "application/vnd.plotly.v1+json": fig_json,
            },
            "id": uid,
        }
    elif mark in ("execute_result_html", "display_html"):
        return {
            "type": "html",
            "data": {
                "text/html": output,
            },
            "id": uid,
        }
    elif mark in ("execute_result_json", "display_json"):
        return {
            "type": "json",
            "data": output,
            "id": uid,
        }
    elif mark in ("execute_result_table", "display_result_table"):
        data = {}
        for k in output:
            if k in [MIME_TABLE_V1, MIME_TABLE_V2]:
                data[k] = output[k]  # Remove unsupported MIME types
        return {
            "type": "table-json",
            "data": data,
            "id": uid,
        }
    return None


def msg_to_secure_block(msg: Optional[dict]):
    if not msg:
        return None
    mark = msg["type"]
    output = msg["output"]
    uid = random.randint(100000, 999999)  # Generate a random UID for the block (6 digits)
    if mark in ("stdout", "execute_result_text", "display_text"):  # ‘error’
        return {
            "type": "text",
            "text": output,
            # "id": uid,
        }
    elif mark in ("execute_result_png", "display_png", "execute_result_jpeg", "display_jpeg"):
        return {
            "type": "image_url",
            "image_url": {
                "url": output,
            },
            "id": uid,
        }
    elif mark in ("execute_result_plotly", "display_plotly"):
        fig_json = msg["fig_json"]
        return {
            "type": "plotly-json",
            "data": {
                "application/vnd.plotly.v1+json": fig_json,
            },
            "id": uid,
        }
    elif mark in ("execute_result_html", "display_html"):
        return {
            "type": "html",
            "data": {
                "text/html": output,
            },
            "id": uid,
        }
    elif mark in ("execute_result_json", "display_json"):
        return {
            "type": "json",
            "data": output,
            "id": uid,
        }
    return None


def parse_msg_list_to_block_list(msg_list: list, mode: Literal["simple", "full"] = "full") -> list[Block]:
    if mode == "simple":
        return list(x for x in map(msg_to_secure_block, msg_list) if x is not None)
    elif mode == "full":
        return list(x for x in map(msg_to_full_block, msg_list) if x is not None)
    return list(x for x in map(msg_to_full_block, msg_list) if x is not None)
